﻿using System;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Input;
using System.Windows.Shapes;


namespace Dx.Controls
{
    public enum ResizingType
    {
        LeftTop,
        Top,
        RightTop,
        Left, 
        Right,
        LeftBottom,
        Bottom, 
        RightBottom
    }

    //Resizable Contorol in Canvas. 
    public class ResizableBorder : ContentControl
    {
        public static readonly DependencyProperty CanResizeProperty = DependencyProperty.Register("CanResize", typeof(bool), typeof(ResizableBorder)
                                                                        , new PropertyMetadata(true));


        [System.Runtime.InteropServices.DllImportAttribute("user32.dll", EntryPoint = "SetCursorPos")]
        [return: System.Runtime.InteropServices.MarshalAsAttribute(System.Runtime.InteropServices.UnmanagedType.Bool)]
        public static extern bool SetCursorPos(int X, int Y);

        public bool CanResize
        {
            get { return (bool)GetValue(CanResizeProperty); }
            set { SetValue(CanResizeProperty, value); }
        }

        private bool IsResizing { get; set; }
        private ResizingType ResizingType { get; set; }
        private Point CurrentPosition { get; set; }
        private Canvas ParentAsCanvas
        {
            get;
            set;
        }
        static ResizableBorder()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(ResizableBorder), new FrameworkPropertyMetadata(typeof(ResizableBorder)));
        }

        public ResizableBorder()
        {
            this.MouseLeave += new MouseEventHandler(ResizableBorder_MouseLeave);
        }

        void ResizableBorder_MouseLeave(object sender, MouseEventArgs e)
        {
            Cursor = null;
        }

        void ResizableBorder_MouseEnter(object sender, MouseEventArgs e)
        {
            if (CanResize == false) return;
            Rectangle arrowRectangle = sender as Rectangle;
            ResizingType resizingType = (ResizingType)Enum.Parse(typeof(ResizingType), arrowRectangle.Name);
            SetCursor(resizingType);
        }

        private void SetCursor(ResizingType resizingType)
        {
            switch (resizingType)
            {
                case ResizingType.Left:
                case ResizingType.Right:
                    Cursor = Cursors.SizeWE;
                    break;
                case ResizingType.Top:
                case ResizingType.Bottom:
                    Cursor = Cursors.SizeNS;
                    break;
                case ResizingType.LeftTop:
                case ResizingType.RightBottom:
                    Cursor = Cursors.SizeNWSE;
                    break;
                case ResizingType.RightTop:
                case ResizingType.LeftBottom:
                    Cursor = Cursors.SizeNESW;
                    break;
            }
        }

        
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            ParentAsCanvas = this.FindParentInVisualTreeNode<Canvas>(5);
            if(ParentAsCanvas == null)
            {
                throw new InvalidOperationException("Parent of ResizableBorder is not Canvas!");
            }
            this.SelectObjects<Rectangle>().ForEach(s=> s.MouseEnter +=new MouseEventHandler(ResizableBorder_MouseEnter));
            this.SelectObjects<Rectangle>().ForEach(s=>s.MouseLeave +=new MouseEventHandler(ResizableBorder_MouseLeave));
            AddHandler(UIElement.MouseDownEvent, new MouseButtonEventHandler(ResizableBorder_PreviewMouseLeftButtonDown), true);
        }
     

        void ResizableBorder_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (!CanResize) return;
            if (e.OriginalSource is Rectangle)
            {
                Rectangle rectangle = e.OriginalSource as Rectangle;
                if (rectangle.Tag != typeof(ResizingType)) return;
                IsResizing = true;
                ResizingType = (ResizingType)Enum.Parse(typeof(ResizingType), rectangle.Name);
                CurrentPosition = e.GetPosition(ParentAsCanvas);
                Application.Current.MainWindow.MouseMove += new MouseEventHandler(ResizableBorder_MouseMove);
                Application.Current.MainWindow.MouseLeftButtonUp += new System.Windows.Input.MouseButtonEventHandler(ResizableBorder_MouseLeftButtonUp);
            }
        }

        void ResizableBorder_MouseMove(object sender, MouseEventArgs e)
        {
            Point newPosition = e.GetPosition(ParentAsCanvas);
            bool needMouseCapture = newPosition.X < 0 || newPosition.Y < 0 
                                    || newPosition.X > ParentAsCanvas.ActualWidth || newPosition.Y > ParentAsCanvas.ActualHeight;
            if (needMouseCapture)
            {
                CaptureMousePos(newPosition);
                return;
            }

            double offsetX = newPosition.X - CurrentPosition.X;
            double offsetY = newPosition.Y - CurrentPosition.Y;

            if (IsResizing == false) return;

            double widthByRightBehavior= this.ActualWidth + offsetX < MinWidth ? MinWidth : this.ActualWidth + offsetX;
            double widthByLeftBehavior= this.ActualWidth - offsetX < MinWidth ? MinWidth : this.ActualWidth - offsetX;
            double heightByTopBehavior= this.ActualHeight - offsetY < MinHeight ? MinHeight : this.ActualHeight - offsetY;
            double heightByBottomBehavior= this.ActualHeight + offsetY<MinHeight?MinHeight:this.ActualHeight +offsetY;

            double newX = newPosition.X, newY = newPosition.Y;
            switch (ResizingType)
            {
                case ResizingType.LeftTop:
                    newX = LeftOperation(widthByLeftBehavior, newPosition);
                    newY = TopOperation(heightByTopBehavior, newPosition);
                    break;
                case ResizingType.Top:
                    newY = TopOperation(heightByTopBehavior, newPosition);
                    break;
                case ResizingType.RightTop:
                    newY = TopOperation(heightByTopBehavior, newPosition);
                    newX = RightOperation(widthByRightBehavior, newPosition);
                    break;
                case ResizingType.Left:
                    newX = LeftOperation(widthByLeftBehavior, newPosition);
                    break;
                case ResizingType.Right:
                    newX = RightOperation(widthByRightBehavior, newPosition);
                    break;
                case ResizingType.LeftBottom:
                    newX = LeftOperation(widthByLeftBehavior, newPosition);
                    newY = BottomOperation(heightByBottomBehavior, newPosition);
                    break;
                case ResizingType.Bottom:
                    newY = BottomOperation(heightByBottomBehavior, newPosition);
                    break;
                case ResizingType.RightBottom:
                    newX = RightOperation(widthByRightBehavior, newPosition);
                    newY = BottomOperation(heightByBottomBehavior, newPosition);
                    break;
            }

            CurrentPosition = new Point(newX,newY);

        }

        private double RightOperation(double widthByRightBehavior, Point newPosition)
        {
            this.Width = widthByRightBehavior;
            if (Width != MinWidth)
            {
                return newPosition.X;
            }
            return Canvas.GetLeft(this) + MinWidth;
        }

        private double TopOperation(double heightByTopBehavior, Point newPosition)
        {
            this.Height = heightByTopBehavior;
            if (Height != MinHeight)
            {
                Canvas.SetTop(this, newPosition.Y);
                return newPosition.Y;
            }
            return CurrentPosition.Y;
        }

        private double BottomOperation(double heightByBottomBehavior, Point newPosition)
        {
            this.Height = heightByBottomBehavior;
            if (Height != MinHeight)
            {
                return newPosition.Y;
            }
            return Canvas.GetTop(this) + MinHeight;
        }

        private double LeftOperation(double widthByLeftBehavior, Point newPosition)
        {
            this.Width = widthByLeftBehavior;
            if (Width != MinWidth)
            {
                Canvas.SetLeft(this, newPosition.X);
                return newPosition.X;
            }
            return CurrentPosition.X;
        }

        private void CaptureMousePos(Point newPosition)
        {
            Point pointToScreen = ParentAsCanvas.PointToScreen(new Point(0, 0));
            if (newPosition.X < 0)
            {
                SetCursorPos((int)pointToScreen.X, (int)(pointToScreen.Y + newPosition.Y));
                return;
            }
            if (newPosition.Y < 0)
            {
                SetCursorPos((int)(pointToScreen.X + newPosition.X), (int)(pointToScreen.Y));
                return;
            }
            if (newPosition.X > ParentAsCanvas.ActualWidth)
            {
                SetCursorPos((int)(pointToScreen.X + ParentAsCanvas.ActualWidth), (int)(pointToScreen.Y + newPosition.Y));
                return;
            }
            if (newPosition.Y > ParentAsCanvas.ActualHeight)
            {
                SetCursorPos((int)(pointToScreen.X + newPosition.X), (int)(pointToScreen.Y + ParentAsCanvas.ActualHeight));
                return;
            }

        }

        void ResizableBorder_MouseLeftButtonUp(object sender, MouseEventArgs e)
        {
            Application.Current.MainWindow.MouseMove -= new MouseEventHandler(ResizableBorder_MouseMove);
            Application.Current.MainWindow.MouseLeftButtonUp -= new System.Windows.Input.MouseButtonEventHandler(ResizableBorder_MouseLeftButtonUp);
            IsResizing = false;
        }
    }
}
