﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Media;
using System.Windows.Data;
using System.Windows;
using System.Windows.Controls;
namespace Dx.Controls
{
    /// <summary>
    /// VisualTree Helper
    /// </summary>
    public static class WpfTreeHelper
    {
        /// <summary>
        ///  visualTree내의 Type에 일치하는 Binding 객체의 Target을 reflesh한다.
        /// </summary>
        /// <param name="visual">root visual 객체</param>
        /// <param name="type">Reflesh할 Type</param>
        /// <param name="dependencyProperty">Reflesh할 Dependency Property</param>
        public static void UpdateBindingTarget(Visual visual, Type type, DependencyProperty dependencyProperty)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(visual); i++)
            {
                Visual childVisual = (Visual)VisualTreeHelper.GetChild(visual, i);

                if (childVisual.GetType().Equals(type))
                {
                    BindingExpression bindExpression = BindingOperations.GetBindingExpression(childVisual, dependencyProperty);
                    if (bindExpression != null)
                        bindExpression.UpdateTarget();
                }
                else
                {
                    UpdateBindingTarget(childVisual, type, dependencyProperty);
                }
            }
        }


        public static void UpdateBindingSource(Visual visual, Type type, DependencyProperty dependencyProperty)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(visual); i++)
            {
                Visual childVisual = (Visual)VisualTreeHelper.GetChild(visual, i);

                if (childVisual.GetType().Equals(type))
                {
                    BindingExpression bindExpression = BindingOperations.GetBindingExpression(childVisual, dependencyProperty);
                    if (bindExpression != null)
                        bindExpression.UpdateSource();
                }
                else
                {
                    UpdateBindingSource(childVisual, type, dependencyProperty);
                }
            }
        }



        /// <summary>
        /// Parent 중 Type "T"와 일치하는 Object를 반환한다. 
        /// </summary>
        /// <typeparam name="T">찾고자 하는 Object의 Type</typeparam>
        /// <param name="current">현재 노드</param>
        /// <param name="depth">Tree Depth를 지정를 지정</param>
        /// <returns>T</returns>
        public static T FindParentInVisualTreeNode<T>(this DependencyObject current, int depth) where T : DependencyObject
        {
            if (depth == 0) return default(T);
            DependencyObject obj = VisualTreeHelper.GetParent(current);
            if (obj == null) return default(T); ;
            if (obj is T)
            {
                return (T)obj;
            }
            else
            {
                return FindParentInVisualTreeNode<T>(obj, depth - 1);
            }
        }


        /// <summary>
        /// ItemsControl 하위 노드의 지정된 Type과 일치하는 Element들을 반환한다.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="current"></param>
        /// <returns></returns>
        public static IList<T> FindObjectInTreeNode<T>(ItemsControl current)
        {
            IList<T> findObjects = new List<T>();

            foreach (object obj in current.Items)
            {
                if (typeof(T).Equals(obj.GetType()))
                {
                    findObjects.Add((T)obj);
                }
                else if (obj is ItemsControl)
                {
                    IList<T> chilredOfObj = FindObjectInTreeNode<T>(obj as ItemsControl);
                    if (chilredOfObj.Count > 0)
                    {
                        foreach (T t in chilredOfObj)
                        {
                            findObjects.Add(t);
                        }
                    }
                }
            }
            return findObjects;
        }


        /// <summary>
        /// SelectObject "T"와 동일한 Type의 Object들을 하위 Tree에서 찾아 List로 만든다.
        /// </summary>
        /// <typeparam name="T">Type</typeparam>
        /// <param name="current">현재 Node</param>
        /// <returns>List</returns>
        public static List<T> SelectObjects<T>(this DependencyObject current) where T : DependencyObject
        {
            List<T> elements = new List<T>();
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(current); i++)
            {
                DependencyObject obj = VisualTreeHelper.GetChild(current, i);
                if (obj is T)
                {
                    elements.Add(obj as T);
                }
                elements.AddRange(obj.SelectObjects<T>());
            }
            return elements;
        }
    }
}
